#ifndef _TASKS_H_
#define _TASKS_H_

#include <avr/io.h>
#include <avr/interrupt.h>

// undefine to save memory if signal & wait are not needed
#define TASK_USE_SIGNAL

// supported / tested devices
#if defined (__AVR_ATmega8__)
#	define RAMSTART 	0x60
#	define STACK_SIZE	64
#	define TIMER_REG	TCCR0
#	define TIMER_ENABLE	TIMSK
#	define TIMER_VECTOR	TIMER0_OVF_vect
#	define CLK1			_BV(CS00)
#	define CLK1024		_BV(CS02)|_BV(CS00)
#elif defined (__AVR_ATmega16__)
#	define RAMSTART 	0x60
#	define STACK_SIZE	64
#	define TIMER_REG	TCCR0
#	define TIMER_ENABLE	TIMSK
#	define TIMER_VECTOR	TIMER0_OVF_vect
#	define CLK1			_BV(CS00)
#	define CLK1024		_BV(CS02)|_BV(CS00)
#elif defined (__AVR_ATmega32__)
#	define RAMSTART 	0x60
#	define STACK_SIZE	96
#	define TIMER_REG	TCCR0
#	define TIMER_ENABLE	TIMSK
#	define TIMER_VECTOR	TIMER0_OVF_vect
#	define CLK1			_BV(CS00)
#	define CLK1024		_BV(CS02)|_BV(CS00)
#elif defined (__AVR_ATmega128__)
#	define RAMSTART 	0x100
#	define STACK_SIZE	128
#	define TIMER_REG	TCCR0
#	define TIMER_ENABLE	TIMSK
#	define TIMER_VECTOR	TIMER0_OVF_vect
#	define CLK1			_BV(CS00)
#	define CLK1024		_BV(CS02)|_BV(CS01)|_BV(CS00)
#elif defined (__AVR_ATmega2560__)
#	define RAMSTART 	0x200
#	define STACK_SIZE	128
#	define TIMER_REG	TCCR0B
#	define TIMER_ENABLE	TIMSK0
#	define TIMER_VECTOR	TIMER0_OVF_vect
#	define CLK1			_BV(CS00)
#	define CLK1024		_BV(CS02)|_BV(CS00)
#elif defined (__AVR_ATtiny44__)
#	define RAMSTART 	0x60
#	define STACK_SIZE	50
#	define TIMER_REG	TCCR0B
#	define TIMER_ENABLE	TIMSK0
#	define TIMER_VECTOR	TIM0_OVF_vect
#	define CLK1			_BV(CS00)
#	define CLK1024		_BV(CS02)|_BV(CS00)
#else
#	error "unsupported device"
#endif

// max number of tasks (depending on ram size)
#define MAX_TASKS ((RAMEND-RAMSTART-100)/(STACK_SIZE+2))

typedef void (*TaskFunction)();

// times & cycles at 16MHz and CLK/1024:
typedef enum TaskPrio
{
	TaskPrioLowLow		= 0xff,	//    64 us :     1024 cycles
	TaskPrioLow			= 0xe0,	//  2048 us :  32*1024 cycles
	TaskPrioMedium		= 0xc0,	//  4096 us :  64*1024 cycles
	TaskPrioHigh		= 0xa0,	//  6144 us :  96*1024 cycles
	TaskPrioHighHigh	= 0x00	// 16384 us : 256*1024 cycles
} TaskPrio;

#ifdef TASK_USE_SIGNAL
typedef enum TaskState
{
	TaskStateActive = 0, 
	TaskStateWaiting = 1

} TaskState;
#endif

typedef volatile struct Task
{
	volatile unsigned char sreg;
	volatile unsigned char *sp;
	volatile TaskPrio prio;
#ifdef TASK_USE_SIGNAL
	volatile TaskState state;
#endif
	volatile unsigned char stack[STACK_SIZE];
} Task;

// create task & add to task-list
void taskCreate(Task *t, TaskFunction f, TaskPrio prio);

// trigger a task switch
// if called from the main thread it starts the scheduler
void taskSwitch();

#ifdef TASK_USE_SIGNAL
typedef struct TaskSignal
{
	Task *waitTask;
} TaskSignal;

// signal & wait functions
void taskSignal(TaskSignal *s);
void taskWait(TaskSignal *s);
#endif

#endif
